﻿using System;
using System.Collections.Generic;
using System.Web.Security;
using System.Text.RegularExpressions;
using System.Web;

namespace MonoSoftware.MonoX
{
    /// <summary>
    /// MonoX Membership provider based on the <see cref="System.Web.Security.SqlMembershipProvider"/> which used <see cref="System.Web.HttpContext"/> items to cache the data on the <see cref="System.Web.HttpRequest"/> base.
    /// </summary>
    public class MonoXMembershipProvider : SqlMembershipProvider
    {
        /// <summary>
        /// Gets the http context cached information from the data source for the membership user associated
        ///     with the specified unique identifier and updates the last activity date/time
        ///     stamp for the user, if specified.
        /// </summary>
        /// <param name="providerUserKey">The unique identifier for the user.</param>
        /// <param name="userIsOnline">true to update the last-activity date/time stamp for the specified user;
        ///     otherwise, false.</param>
        /// <returns> A System.Web.Security.MembershipUser object representing the user associated
        ///     with the specified unique identifier. If no user is found in the database
        ///     for the specified providerUserKey value, null is returned.</returns>
        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            if (providerUserKey == null)
                return null;
            string key = String.Format("GetUser-providerUserKey-{0}-{1}", providerUserKey, userIsOnline);
            MembershipUser user = null;
            if (!MonoXContextManager.HttpContextItemExists(key))
            {
                user = base.GetUser(providerUserKey, userIsOnline);
                if (user != null)
                    MonoXContextManager.SetHttpContextItem(key, user);
            }
            else
                user = MonoXContextManager.GetHttpContextItem<MembershipUser>(key);
            return user;
        }

        /// <summary>
        /// Returns http context cached information from the SQL Server membership database for a user and
        ///     provides an option to update the last activity date/time stamp for the user.
        /// </summary>
        /// <param name="username">The name of the user to get information for.</param>
        /// <param name="userIsOnline">true to update the last activity date/time stamp for the user; false to return
        ///     user information without updating the last activity date/time stamp for the
        ///     user.</param>
        /// <returns>A System.Web.Security.MembershipUser object representing the specified user.
        ///     If no user is found in the database for the specified username value, null
        ///     is returned.</returns>
        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            if (String.IsNullOrEmpty(username))
                return null;
            string key = String.Format("GetUser-username-{0}-{1}", username, userIsOnline);
            MembershipUser user = null;
            if (!MonoXContextManager.HttpContextItemExists(key))
            {
                user = base.GetUser(username, userIsOnline);
                if (user != null)
                    MonoXContextManager.SetHttpContextItem(key, user);
            }
            else
                user = MonoXContextManager.GetHttpContextItem<MembershipUser>(key);
            return user;
        }

        /// <summary>
        /// Gets the http context cached user name associated with the specified e-mail address.
        /// </summary>
        /// <param name="email">The e-mail address to search for.</param>
        /// <returns>The user name associated with the specified e-mail address. If no match is
        ///     found, this method returns null.</returns>
        public override string GetUserNameByEmail(string email)
        {
            if (String.IsNullOrEmpty(email))
                return null;
            string key = String.Format("GetUserNameByEmail-email-{0}", email);
            string username = String.Empty;
            if (!MonoXContextManager.HttpContextItemExists(key))
            {
                username = base.GetUserNameByEmail(email);
                if (!String.IsNullOrEmpty(username))
                    MonoXContextManager.SetHttpContextItem(key, username);
            }
            else
                username = MonoXContextManager.GetHttpContextItem<string>(key);
            return username;
        }

        /// <summary>
        /// Gets a http context cached collection of membership users where the user name contains the specified
        ///     user name to match.
        /// </summary>
        /// <param name="usernameToMatch">The user name to search for.</param>
        /// <param name="pageIndex">The index of the page of results to return. pageIndex is zero-based.</param>
        /// <param name="pageSize">The size of the page of results to return.</param>
        /// <param name="totalRecords">When this method returns, contains the total number of matched users.</param>
        /// <returns>A System.Web.Security.MembershipUserCollection that contains a page of pageSizeSystem.Web.Security.MembershipUser
        ///     objects beginning at the page specified by pageIndex.</returns>
        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            KeyValuePair<int, MembershipUserCollection> pair = new KeyValuePair<int, MembershipUserCollection>();
            string key = String.Format("FindUsersByName-usernameToMatch-{0}-{1}-{2}", usernameToMatch, pageIndex, pageSize);
            if (!MonoXContextManager.HttpContextItemExists(key))
            {
                MembershipUserCollection users = base.FindUsersByName(usernameToMatch, pageIndex, pageSize, out totalRecords);
                pair = new KeyValuePair<int, MembershipUserCollection>(totalRecords, users);
                if (totalRecords > 0)
                    MonoXContextManager.SetHttpContextItem(key, pair);
            }
            else
                pair = MonoXContextManager.GetHttpContextItem<KeyValuePair<int, MembershipUserCollection>>(key);
            totalRecords = pair.Key;
            return pair.Value;
        }

        /// <summary>
        ///  Removes a user's membership information from the SQL Server membership database.
        /// </summary>
        /// <param name="username">The name of the user to delete.</param>
        /// <param name="deleteAllRelatedData">true to delete data related to the user from the database; false to leave
        ///     data related to the user in the database.</param>
        /// <returns> true if the user was deleted; otherwise, false. A value of false is also
        ///     returned if the user does not exist in the database.</returns>
        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            bool result = base.DeleteUser(username, deleteAllRelatedData);
            if (result)
                MonoSoftware.Web.Security.Membership.InvalidateProviderUserKey(username);
            return result;
        }

        public override string GeneratePassword()
        {
            string password = base.GeneratePassword();
            if (!String.IsNullOrWhiteSpace(PasswordStrengthRegularExpression))
            {
                int counter = 0;
                while (!Regex.IsMatch(password, PasswordStrengthRegularExpression) && (counter <= ApplicationSettings.PasswordGenerationRetryCount))
                {
                    password = base.GeneratePassword();
                    counter++;
                }                
            }
            return password;
        }

        //Sumit Start
        /// <summary>
        /// Validator css class used to improve validator appearance.
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                HttpContext context = HttpContext.Current;
                if (context == null)
                {
                    throw new InvalidOperationException("Http context cannot be null.");
                }

                string applicationName = String.Empty;
                if (context.Items.Contains(ApplicationSettings.ApplicationName))
                {
                    if (!String.IsNullOrEmpty((string)
                        context.Items[ApplicationSettings.ApplicationName]))
                    {
                        applicationName = (string)
                            context.Items[ApplicationSettings.ApplicationName];
                    }
                }
                return applicationName;
            }
            set
            {
                base.ApplicationName = value;
            }
        }
        //Sumit End        
    }
}