using MonoSoftware.MonoX.Caching;
using MonoSoftware.MonoX.Repositories;
using MonoSoftware.MonoX.Utilities;
using System;
using System.Collections.Specialized;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace MonoSoftware.MonoX
{
    /// <summary>
    /// Enhanced version of the SqlPersonalizationProvider that introduces versioning features to portal.
    /// </summary>
    public class VersionedPersonalizationProvider : SqlPersonalizationProvider
    {

        /// <summary>
        /// Used to vary personalization by query string parameters.
        /// </summary>
        private string[] _varyByParams = null;
        /// <summary>
        /// Used to vary personalization by custom parameters.
        /// </summary>
        private string _varyByCustom = String.Empty;

        private string _rootPagePath;
        /// <summary>
        /// Root page path, before varyByParams or varyByCustom are applied to it.
        /// </summary>
        public string RootPagePath
        {
            get { return _rootPagePath; }
            set { _rootPagePath = value; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public VersionedPersonalizationProvider() { }

        /// <summary>
        /// Loads personalization blobs.
        /// </summary>
        /// <param name="wpm">Web part manager</param>
        /// <param name="path">Path</param>
        /// <param name="userName">Username</param>
        /// <param name="sharedDataBlob">Shared data blob</param>
        /// <param name="userDataBlob">User data blob</param>
        protected override void LoadPersonalizationBlobs(WebPartManager wpm, string path, string userName, ref byte[] sharedDataBlob, ref byte[] userDataBlob)
        {

            path = GetPath;
            PersonalizationStateQuery query = new PersonalizationStateQuery();
            query.PathToMatch = path + "|revision";

            bool pageHasRevisionData = GetCountOfState(PersonalizationScope.Shared, query) > 0;
            bool pageHasRevisionDocuments = GetCountOfDocuments(PersonalizationScope.Shared, query) > 0;

            bool isRevisionPage = ((PortalWebPartManager)wpm).IsRevisionPage;
            //since the LoadPersonalizationBlobs event happens very early, there is no time to set the revision state on the first occasion; therefore, check if the postback was caused by the revision button
            //DO NOT use MonoSoftware.Web.PageExtension.GetPostBackControl to find the control that has caused a postback, since it enumerates the controls in the wrong order and causes CreateChildControls to fire before OnInit on editor parts and catalogs
            if (PageUtility.IsPostbackCausedByButton(".*PageTasks.*btnRevision"))
                isRevisionPage = !isRevisionPage;
            
            ((PortalWebPartManager)wpm).HasUnapprovedChanges = pageHasRevisionData;
            ((PortalWebPartManager)wpm).HasUnapprovedDocuments = pageHasRevisionDocuments;

            if (isRevisionPage && pageHasRevisionData)
            {
                path += "|revision";
            }

            base.LoadPersonalizationBlobs(
                wpm,
                path,
                userName,
                ref sharedDataBlob,
                ref userDataBlob
            );

        }

        /// <summary>
        /// Resets personalization blob.
        /// </summary>
        /// <param name="wpm">Web part manager</param>
        /// <param name="path">Path</param>
        /// <param name="userName">Username</param>
        protected override void ResetPersonalizationBlob(WebPartManager wpm, string path, string userName)
        {
            if (!SecurityUtility.AllowPersonalization()) return;

            base.ResetPersonalizationBlob(wpm, path, userName);
            //base.ResetPersonalizationBlob(wpm, this.GetPath, userName);
        }

        /// <summary>
        /// Saves personalization blob.
        /// </summary>
        /// <param name="wpm">Web part manager</param>
        /// <param name="path">Path</param>
        /// <param name="userName">Username></param>
        /// <param name="dataBlob">Data blob</param>
        protected override void SavePersonalizationBlob(WebPartManager wpm, string path, string userName, byte[] dataBlob)
        {
            if (!SecurityUtility.AllowPersonalization()) return;

            path = GetPath;
            bool isRevisionPage =
                ((PortalWebPartManager)wpm).IsRevisionPage;

            if (((PortalWebPartManager)wpm).CommittingChanges)
            {

                base.SavePersonalizationBlob(
                    wpm, path, userName, dataBlob);

                ResetPersonalizationBlob(
                    wpm, path + "|revision", null);

                //take care of the revised documents
                //the code below does not save documents on the rewritten pages
                if (wpm.Page is BasePage)
                    DocumentRepository.GetInstance().SaveApprovedDocuments((wpm.Page as BasePage).PageProperties.Id, string.Empty, userName);
                else
                    DocumentRepository.GetInstance().SaveApprovedDocuments(Guid.Empty, path, userName);
                ((PortalWebPartManager)wpm).HasUnapprovedChanges = false;
            }
            else if (isRevisionPage)
            {

                base.SavePersonalizationBlob(
                    wpm,
                    path + "|revision",
                    userName,
                    dataBlob);

            }
            else
            {
                base.SavePersonalizationBlob(
                    wpm,
                    path,
                    userName,
                    dataBlob);
            }
        }

        //The code below is taken from http://fredrik.nsquared2.com/viewpost.aspx?PostID=326
        //The WebPart personalization providers shipped with ASP.Net 2.0, will by default use the URL as an identifier to get and store personalized data, but it will not include querystrings. This provider can include querystrings. To specify which querystrings that should be used, you simply add the varyByParams attribute to the provider, and use a comma separated list with the querystring parameters that should be included. If you want to include all querysteings, just set the varyByParams to "none".
        //<webParts>
        //   <personalization defaultProvider="Nsquared2SqlPersonalizationProvider">
        //      <providers>
        //         <add
        //             name="Nsquared2SqlPersonalizationProvider"
        //             type="Nsquared2.Web.UI.WebControls.WebParts.SqlPersonalizationProvider, Nsquared2.Web"
        //             connectionStringName="LocalSqlServer"
        //             varyByParams="page">
        //                </add>
        //            </providers>
        //        </personalization>
        //        </webParts>

        /// <summary>
        /// Initialization.
        /// </summary>
        /// <param name="name">Name</param>
        /// <param name="configSettings">Configuration settings</param>
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection configSettings)
        {
            string varyByParams = configSettings["varyByParams"];
            if (!string.IsNullOrEmpty(varyByParams))
                _varyByParams = MonoXUtility.SplitConfigurationString(varyByParams);
            configSettings.Remove("varyByParams");

            string varyByCustom = configSettings["varyByCustom"];
            if (!string.IsNullOrEmpty(varyByCustom))
                _varyByCustom = GetVaryByCustomString(varyByCustom);
            configSettings.Remove("varyByCustom");

            base.Initialize(name, configSettings);
        }

        public virtual string GetVaryByCustomString(string varyByCustom)
        {
            string[] keys = MonoXUtility.SplitConfigurationString(varyByCustom);
            string toReturn = string.Empty;
            foreach (string key in keys)
            {
                switch (key.ToLower())
                {
                    case "languageid" :
                        toReturn = string.Format("{0}|lid={1}", toReturn, MonoSoftware.Core.ShortGuid.Encode(LocalizationUtility.GetCurrentLanguageId()));
                        break;
                    default:
                        break;
                }
            }
            return toReturn;
        }

        /// <summary>
        /// Finds the current personalization path.
        /// </summary>
        public virtual string GetPath
        {
            get
            {
                HttpContext context = HttpContext.Current;

                if (context == null)
                    throw new HttpException("Context can not be Null");

                HttpRequest request = context.Request;
                if (request == null)
                    throw new HttpException("The current Context's Request property can not be Null");

                return CreateUrl(context, request);
            }
        }

        /// <summary>
        /// Creates personalization URL that is used to uniqely identify the page to personalize.
        /// </summary>
        /// <param name="context">Current HTTP context.</param>
        /// <param name="request">Request.</param>
        /// <returns>Formatted URL.</returns>
        private string CreateUrl(HttpContext context, HttpRequest request)
        {
            NameValueCollection parameters = request.QueryString;
            

            //since RawUrl is not set while context.RewritePath executes, and it holds the value of the calling page instead of the rewritten page
            //so check if the session variable is set
            string path = String.Empty;
            bool personalizeByRawUrl = false;
            string urlKey = String.Empty;
            //07.10.2010 introduced personalization by absolutepath, which is a default method. Personalization by raw URL is still supported, but the user has to set the property on the page to use it
            //personaliztion by raw URL does not work for container modules (blog, groups, discusion) that heavily use URL rewriting - if a container part is dynamically instantiated it dissapears when user follows a link that utilizes rewriting (for example, view blog post), since this generates a new raw URL and engine cannot find the personalization information for that URL, hence the page is empty
            if (context.Handler != null && context.Handler is BasePage)
                personalizeByRawUrl = ((BasePage)context.Handler).PersonalizeByRawUrl;
                    
            if (personalizeByRawUrl)
                urlKey = request.RawUrl;
            else
                urlKey = request.Url.AbsolutePath;

            if (context.Session != null && context.Session["PageExecutionRawUrl"] != null)
            {
                path = context.Session["PageExecutionRawUrl"].ToString();
                context.Session.Remove("PageExecutionRawUrl");
            }
            //Remove query params
            else
            {
                if (urlKey.IndexOf('?') != -1)
                    path = urlKey.Remove(urlKey.IndexOf('?'));
                else
                    path = urlKey;
                //Remove Application path if any
                if (request.ApplicationPath != "/")
                    path = path.Substring(path.ToLower().IndexOf(request.ApplicationPath.ToLower()) + request.ApplicationPath.Length);
                path = String.Format("~{0}", path);
            }
            RootPagePath = path;
            return GeneratePersonalizationUrl(path, parameters);
        }

        /// <summary>
        /// Generates URL in the format that is suitable to be used in the personalization system, taking into account varyByParams and varyByCustom settings.
        /// </summary>
        /// <param name="url">URL to format.</param>
        /// <returns>Formatted URL.</returns>
        public string GeneratePersonalizationUrl(string url)
        {
            UriBuilder builder = new UriBuilder(MonoSoftware.Web.UrlFormatter.ResolveServerUrl(url));
            string path = MonoSoftware.Web.UrlFormatter.ToRelativeUrl(builder.Path);
            NameValueCollection parameters = HttpUtility.ParseQueryString(builder.Query);
            return GeneratePersonalizationUrl(path, parameters);
        }

        /// <summary>
        /// Generates URL in the format that is suitable to be used in the personalization system, taking into account varyByParams and varyByCustom settings.
        /// </summary>
        /// <param name="path">Path part of the full URL.</param>
        /// <param name="parameters">Query parameters.</param>
        /// <returns>Formatted URL.</returns>
        protected string GeneratePersonalizationUrl(string path, NameValueCollection parameters)
        {
            if ((_varyByParams != null) && (_varyByParams.Length > 0))
            {
                if (_varyByParams[0].ToLower() != "none")
                {

                    for (int paramIndex = 0; paramIndex < _varyByParams.Length; paramIndex++)
                    {
                        string paramValue = parameters[_varyByParams[paramIndex]] as string;

                        if (!string.IsNullOrEmpty(paramValue))
                        {
                            if (paramIndex == 0)
                                path = string.Concat(path, "?");
                            else
                                path = string.Concat(path, "&");

                            path = string.Concat(path, _varyByParams[paramIndex].Trim(), "=", MonoSoftware.Web.UrlEncoder.UrlDecode(paramValue));
                        }
                    }
                }
                //add varyByCustom if any
                path = path + _varyByCustom;

                return path;
            }
            //add varyByCustom if any
            path = path + _varyByCustom;

            return path;
        }

        /// <summary>
        /// Gets count of state.
        /// </summary>
        /// <param name="scope">Scope.</param>
        /// <param name="query">Query.</param>
        /// <returns>Count of state.</returns>
        public override int GetCountOfState(PersonalizationScope scope, PersonalizationStateQuery query)
        {
            //if ((HttpContext.Current != null) && (HttpContext.Current.Handler is Page))
            //{
            //    Page page = (Page)HttpContext.Current.Handler;
            //    WebPartManager webPartManager = WebPartManager.GetCurrentWebPartManager(page);
            //    if (webPartManager != null)
            //    {
            //        //query.PathToMatch = this.GetPath;
            //        //we are using "full" query below, even if it contains the revision suffix, since GetCountOfState uses original aspnet_paths table
            //        return base.GetCountOfState(scope, query);
            //    }
            //}
            return base.GetCountOfState(scope, query);
        }

        /// <summary>
        /// Gets the count of documents under revision.
        /// </summary>
        /// <param name="scope">Personalization scope.</param>
        /// <param name="query">Personalization state query.</param>
        /// <returns>Count of documents under revision.</returns>
        public int GetCountOfDocuments(PersonalizationScope scope, PersonalizationStateQuery query)
        {
            int toReturn = 0;
            //this is executed on each page access, so cache the value 
            MonoXCacheManager cacheManager = MonoXCacheManager.GetInstance(CacheKeys.Utilities.Documents, MonoSoftware.MonoX.Caching.MonoXCacheManager.DefaultSystemCacheDuration);
            toReturn = cacheManager.Get<int>(RootPagePath, "Revisions");
            if (toReturn == 0)
            {
                if ((HttpContext.Current != null) && (HttpContext.Current.Handler is Page))
                {
                    //do not use query.PathToMatch or GetPath since it may contain "|revision" or varyByParam/varyByCustom suffix if page is under revision (only used for the aspnet_paths table, not for the page table we are using to get documents)
                    toReturn = PageRepository.GetInstance().GetCountOfDocumentsInRevision(RootPagePath);
                    //cacheManager.Get retruns a default value - zero for int type - if the value is not present in the cache
                    //sinc zero is a valid value here, set an invalid value to indicate that a value is present in cache
                    if (toReturn == 0)
                        cacheManager.Store(-1, this.GetPath, "Revisions");
                    else
                        cacheManager.Store(toReturn, this.GetPath, "Revisions");
                }
            }
            else if (toReturn < 0)
                toReturn = 0;
            return toReturn;
        }

        //Sumit Start
        /// <summary>
        /// Validator css class used to improve validator appearance.
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                HttpContext context = HttpContext.Current;
                if (context == null)
                {
                    throw new InvalidOperationException("Http context cannot be null.");
                }

                string applicationName = String.Empty;
                if (context.Items.Contains(ApplicationSettings.ApplicationName))
                {
                    if (!String.IsNullOrEmpty((string)
                        context.Items[ApplicationSettings.ApplicationName]))
                    {
                        applicationName = (string)
                            context.Items[ApplicationSettings.ApplicationName];
                    }
                }
                return applicationName;
            }
            set
            {
                base.ApplicationName = value;
            }
        }
        //Sumit End
        
    }
}